@@ -28,6 +28,7 @@ import unittest
 
 
 import some_module
+from proxy import Proxy
 
 
 def module_level_function(some, args):
@@ -37,9 +38,14 @@ def module_level_function(some, args):
 module_level_attribute = 'test'
 
 class SomeClass:
+    CLASS_VALUE = "class_method"
+
+    def __init__(self):
+        self.instance_value = "instance_method"
+
     @classmethod
     def class_method(cls):
-        return "class_method"
+        return cls.CLASS_VALUE
 
     @classmethod
     def class_method_with_args(cls, a):
@@ -54,7 +60,7 @@ class SomeClass:
         return a
 
     def instance_method(self):
-        return "instance_method"
+        return self.instance_value
 
     def instance_method_with_args(self, a):
         return a
@@ -2158,6 +2164,146 @@ if sys.version_info >= (3, 0):
                     'kwargs_only_func').with_args,
                 1)
 
+        def test_mock_proxied_class(self):
+            # pylint: disable=not-callable
+            SomeClassProxy = Proxy(SomeClass)
+            flexmock(SomeClassProxy).should_receive("class_method").and_return(2).twice()
+            assert SomeClassProxy().class_method() == 2
+            assert SomeClassProxy.class_method() == 2
+            flexmock(SomeClassProxy).should_receive("static_method").and_return(3).twice()
+            assert SomeClassProxy().static_method() == 3
+            assert SomeClassProxy.static_method() == 3
+            instance = SomeClassProxy()
+            flexmock(instance).should_receive("instance_method").and_return(4).once()
+            assert instance.instance_method() == 4
+
+        def test_mock_proxied_class_with_args(self):
+            # pylint: disable=not-callable
+            SomeClassProxy = Proxy(SomeClass)
+            flexmock(SomeClassProxy).should_receive("class_method_with_args").with_args("a").and_return(
+                2
+            ).twice()
+            assert SomeClassProxy().class_method_with_args("a") == 2
+            assert SomeClassProxy.class_method_with_args("a") == 2
+            flexmock(SomeClassProxy).should_receive("static_method_with_args").with_args(
+                "b"
+            ).and_return(3).twice()
+            assert SomeClassProxy().static_method_with_args("b") == 3
+            assert SomeClassProxy.static_method_with_args("b") == 3
+            instance = SomeClassProxy()
+            flexmock(instance).should_receive("instance_method_with_args").with_args("c").and_return(
+                4
+            ).once()
+            assert instance.instance_method_with_args("c") == 4
+
+        def test_spy_proxied_class(self):
+            # pylint: disable=not-callable
+            SomeClassProxy = Proxy(SomeClass)
+            flexmock(SomeClassProxy).should_call("class_method").and_return("class_method").twice()
+            assert SomeClassProxy().class_method() == "class_method"
+            assert SomeClassProxy.class_method() == "class_method"
+            flexmock(SomeClassProxy).should_call("static_method").and_return("static_method").twice()
+            assert SomeClassProxy().static_method() == "static_method"
+            assert SomeClassProxy.static_method() == "static_method"
+            instance = SomeClassProxy()
+            flexmock(instance).should_call("instance_method").and_return("instance_method").once()
+            assert instance.instance_method() == "instance_method"
+
+        def test_spy_proxied_class_with_args(self):
+            # pylint: disable=not-callable
+            SomeClassProxy = Proxy(SomeClass)
+            flexmock(SomeClassProxy).should_call("class_method_with_args").with_args("a").and_return(
+                "a"
+            ).twice()
+            assert SomeClassProxy().class_method_with_args("a") == "a"
+            assert SomeClassProxy.class_method_with_args("a") == "a"
+            flexmock(SomeClassProxy).should_call("static_method_with_args").with_args("b").and_return(
+                "b"
+            ).twice()
+            assert SomeClassProxy().static_method_with_args("b") == "b"
+            assert SomeClassProxy.static_method_with_args("b") == "b"
+            instance = SomeClassProxy()
+            flexmock(instance).should_call("instance_method_with_args").with_args("c").and_return(
+                "c"
+            ).once()
+            assert instance.instance_method_with_args("c") == "c"
+
+        def test_mock_proxied_derived_class(self):
+            # pylint: disable=not-callable
+            DerivedClassProxy = Proxy(DerivedClass)
+            flexmock(DerivedClassProxy).should_receive("class_method").and_return(2).twice()
+            assert DerivedClassProxy().class_method() == 2
+            assert DerivedClassProxy.class_method() == 2
+            flexmock(DerivedClassProxy).should_receive("static_method").and_return(3).twice()
+            assert DerivedClassProxy().static_method() == 3
+            assert DerivedClassProxy.static_method() == 3
+            instance = DerivedClassProxy()
+            flexmock(instance).should_receive("instance_method").and_return(4).once()
+            assert instance.instance_method() == 4
+
+        def test_mock_proxied_module_function(self):
+            # pylint: disable=not-callable
+            some_module_proxy = Proxy(some_module)
+            flexmock(some_module_proxy).should_receive("foo").and_return(3).once()
+            assert some_module_proxy.foo() == 3
+
+        def test_spy_proxied_module_function(self):
+            # pylint: disable=not-callable
+            some_module_proxy = Proxy(some_module)
+            flexmock(some_module_proxy).should_receive("foo").and_return(0).once()
+            assert some_module_proxy.foo(2, 2) == 0
+
+        def test_mock_proxied_derived_class_with_args(self):
+            # pylint: disable=not-callable
+            DerivedClassProxy = Proxy(DerivedClass)
+            flexmock(DerivedClassProxy).should_receive("class_method_with_args").with_args(
+                "a"
+            ).and_return(2).twice()
+            assert DerivedClassProxy().class_method_with_args("a") == 2
+            assert DerivedClassProxy.class_method_with_args("a") == 2
+            flexmock(DerivedClassProxy).should_receive("static_method_with_args").with_args(
+                "b"
+            ).and_return(3).twice()
+            assert DerivedClassProxy().static_method_with_args("b") == 3
+            assert DerivedClassProxy.static_method_with_args("b") == 3
+            instance = DerivedClassProxy()
+            flexmock(instance).should_receive("instance_method_with_args").with_args("c").and_return(
+                4
+            ).once()
+            assert instance.instance_method_with_args("c") == 4
+
+        def test_spy_proxied_derived_class(self):
+            # pylint: disable=not-callable
+            DerivedClassProxy = Proxy(DerivedClass)
+            flexmock(DerivedClassProxy).should_call("class_method").and_return("class_method").twice()
+            assert DerivedClassProxy().class_method() == "class_method"
+            assert DerivedClassProxy.class_method() == "class_method"
+            flexmock(DerivedClassProxy).should_call("static_method").and_return("static_method").twice()
+            assert DerivedClassProxy().static_method() == "static_method"
+            assert DerivedClassProxy.static_method() == "static_method"
+            instance = DerivedClassProxy()
+            flexmock(instance).should_call("instance_method").and_return("instance_method").once()
+            assert instance.instance_method() == "instance_method"
+
+        def test_spy_proxied_derived_class_with_args(self):
+            # pylint: disable=not-callable
+            DerivedClassProxy = Proxy(DerivedClass)
+            flexmock(DerivedClassProxy).should_call("class_method_with_args").with_args("a").and_return(
+                "a"
+            ).twice()
+            assert DerivedClassProxy().class_method_with_args("a") == "a"
+            assert DerivedClassProxy.class_method_with_args("a") == "a"
+            flexmock(DerivedClassProxy).should_call("static_method_with_args").with_args(
+                "b"
+            ).and_return("b").twice()
+            assert DerivedClassProxy().static_method_with_args("b") == "b"
+            assert DerivedClassProxy.static_method_with_args("b") == "b"
+            instance = DerivedClassProxy()
+            flexmock(instance).should_call("instance_method_with_args").with_args("c").and_return(
+                "c"
+            ).once()
+            assert instance.instance_method_with_args("c") == "c"
+
 
 if __name__ == '__main__':
     unittest.main()
